package ea;

import java.util.Random;

public class BitVectorIndividual extends Individual {

	private static final Random RANDOM = new Random();

	private boolean[] genome;

	public BitVectorIndividual(int length) {
		genome = new boolean[length];
		for (int i = 0; i < length; i++) {
			genome[i] = RANDOM.nextBoolean();
		}
	}

	protected BitVectorIndividual(boolean[] genome) {
		this.genome = genome;
	}

	@Override
	public BitVectorIndividual mateWith(Individual partner, double crossover, double mutation) {
		BitVectorIndividual o = (BitVectorIndividual) partner;
		int length = genome.length;
		int split = length;
		if (RANDOM.nextDouble() < crossover) {
			split = RANDOM.nextInt(length);
		}
		boolean[] childGenome = new boolean[length];
		System.arraycopy(genome, 0, childGenome, 0, split);
		System.arraycopy(o.genome, split, childGenome, split, length - split);
		for (int i = 0; i < length; i++) {
			if (RANDOM.nextDouble() < mutation) {
				childGenome[i] = !childGenome[i];
			}
		}
		return newIndividual(childGenome, this);
	}

	protected BitVectorIndividual newIndividual(boolean[] genome, Individual individual) {
		return new BitVectorIndividual(genome);
	}

	public boolean[] getBitVector() {
		return genome;
	}

	@Override
	public String phenomeString() {
		boolean[] bitVector = getBitVector();
		String phenome = "";
		for (int i = 0; i < bitVector.length; i++) {
			phenome += (bitVector[i] ? "1" : "0") + ", ";
		}
		phenome = phenome.substring(0, phenome.length() - 2);
		return phenome;
	}
}
