package ea;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;

import cbr.CBR;
import cbr.Caze;
import cbr.Decision;

public class CBRTester implements FitnessTester {

	private static final int NOF_TESTS = 1;

	private final Set<Caze> cazes;

	public CBRTester(Set<Caze> cazes) {
		this.cazes = cazes;
	}

	@Override
	public void testIndividuals(Set<Individual> individuals) {
		List<Set<Caze>> cazeBases = new LinkedList<>();
		List<Set<Caze>> testSets = new LinkedList<>();
		List<Caze> cazeList = new ArrayList<>(cazes);
		for (int i = 0; i < NOF_TESTS; i++) {
			Collections.shuffle(cazeList);
			int cazeBaseSize = (int) (0.75 * cazeList.size());
			Set<Caze> cazeBase = new HashSet<>(cazeList.subList(0, cazeBaseSize));
			Set<Caze> testSet = new HashSet<>(cazeList.subList(cazeBaseSize, cazeList.size()));
			cazeBases.add(cazeBase);
			testSets.add(testSet);
		}
		for (Individual individual : individuals) {
			ByteArrayIndividual byteArrayIndividual = (ByteArrayIndividual) individual;
			EASimilarityFunction similarityFunction = new EASimilarityFunction(byteArrayIndividual);
			CBR cbr = new CBR(similarityFunction);
			double fitness = 0.0;
			for (int i = 0; i < NOF_TESTS; i++) {
				fitness += testCBR(cbr, cazeBases.get(i), testSets.get(i));
			}
			fitness /= NOF_TESTS;
			fitness = Math.max(fitness, 0.0);
			fitness = Math.min(fitness, 1.0);
			individual.setFitness(fitness);
		}
	}

	private double testCBR(CBR cbr, Set<Caze> cazeBase, Set<Caze> testSet) {
		cbr.setCazeBase(cazeBase);
		int correctOpen = 0;
		int totalOpen = 0;
		int correctConvoy = 0;
		int totalConvoy = 0;
		int correctClosed = 0;
		int totalClosed = 0;
		for (Caze query : testSet) {
			Decision decision = cbr.makeDecision(query);
			switch (query.getDecision()) {
			case OPEN:
				if (decision == Decision.OPEN) {
					correctOpen++;
				}
				totalOpen++;
				break;
			case CONVOY:
				if (decision == Decision.CONVOY) {
					correctConvoy++;
				}
				totalConvoy++;
				break;
			case CLOSED:
				if (decision == Decision.CLOSED) {
					correctClosed++;
				}
				totalClosed++;
				break;
			}
		}
		int nofDecisionTypes = 0;
		double result = 0.0;
		if (totalOpen != 0) {
			nofDecisionTypes++;
			result += (double) correctOpen / (double) totalOpen;
		}
		if (totalConvoy != 0) {
			nofDecisionTypes++;
			result += (double) correctConvoy / (double) totalConvoy;
		}
		if (totalClosed != 0) {
			nofDecisionTypes++;
			result += (double) correctClosed / (double) totalClosed;
		}
		if (nofDecisionTypes == 0) {
			throw new RuntimeException("No test cases");
		}
		result /= nofDecisionTypes;
		return result;
	}
}
