package ea;

import java.util.HashSet;
import java.util.Random;
import java.util.Set;

public class SigmaScalingReproducer implements Reproducer {

	private static final Random RANDOM = new Random();

	private int noChildren;
	private double crossover;
	private double mutation;

	public SigmaScalingReproducer(int noChildren, double crossover, double mutation) {
		this.noChildren = noChildren;
		this.crossover = crossover;
		this.mutation = mutation;
	}

	@Override
	public Set<Individual> generateChildren(EA ea) {
		Set<Individual> adults = ea.getIndividuals();
		double avg = ea.avgFitness();
		double sd = ea.sdFitness();
		Set<Individual> children = new HashSet<Individual>();
		while (children.size() < noChildren) {
			Individual parentA = selectParent(adults, avg, sd);
			Individual parentB = selectParent(adults, avg, sd);
			Individual child = parentA.mateWith(parentB, crossover, mutation);
			children.add(child);
		}
		return children;
	}

	private Individual selectParent(Set<Individual> adults, double avg, double sd) {
		Set<ParentCandidate> candidates = new HashSet<ParentCandidate>();
		double totalProbability = 0;
		for (Individual individual : adults) {
			double probability;
			if (sd != 0) {
				probability = 1 + (individual.getFitness() - avg) / (2 * sd);
			} else {
				probability = individual.getFitness();
			}
			candidates.add(new ParentCandidate(individual, probability));
			totalProbability += probability;
		}
		double random = totalProbability * RANDOM.nextDouble();
		for (ParentCandidate candidate : candidates) {
			random -= candidate.getProbability();
			if (random < 0) {
				return candidate.getIndividual();
			}
		}
		return null;
	}
}
